# -*- coding: utf-8 -*-

from django.test import TestCase
from lxml import etree as ET

from semanticeditor.api import extract_structure, PresentationInfo, format_html, extract_presentation, clean_html, preview_html, get_classes
from semanticeditor.common import html_extract, parse, get_structure
from semanticeditor.definitions import IncorrectHeadings, BadStructure, TooManyColumns, PresentationClass, NEWROW, NEWCOL, NEWINNERROW, NEWINNERCOL
from semanticeditor.models import CssClass
from semanticeditor.layout import LayoutDetails
from semanticeditor.utils.etree import get_index, get_parent, eliminate_tag, indent


PC = PresentationClass


def pretty_print(content):
    t = parse(content)
    indent(t)
    return html_extract(t)


class TestExtractStructure(TestCase):
    def test_extract_structure(self):
        self.assertEqual([(s.level, s.sect_id, s.name, s.tag) for s in extract_structure(u"""
<h1>Heading <b>with </b><i>embedded <em>stuff</em> in</i> it</h1> Hmm
<p>A long paragraph with some actual content</p>
<h2>A sub heading</h2>
<p>Another para</p>
<h3>level 3</h3>
<p>A long paragraph with some actual content</p>
<h4>level 4</h4>
<p>Another para</p>
<h5>level 5</h5>
<p>nasty  éééééééééééééééééééééééééé</p>
<h6>level 6</h6>
<h1>Heading two</h1>
""")],
        [(1, "h1_1", u"Heading with embedded stuff in it", u"h1"),
         (2, "p_1", u"A long paragraph with some actual content...", u"p"),
         (2, "h2_1", u"A sub heading", u"h2"),
         (3, "p_2", u"Another para...", u"p"),
         (3, "h3_1", u"level 3", u"h3"),
         (4, "p_3", u"A long paragraph with some actual content...", u"p"),
         (4, "h4_1", u"level 4", u"h4"),
         (5, "p_4", u"Another para...", u"p"),
         (5, "h5_1", u"level 5", u"h5"),
         (6, "p_5", u"nasty  éééééééééééééééééééééééééé...", u"p"),
         (6, "h6_1", u"level 6", u"h6"),
         (1, "h1_2", u"Heading two", u"h1"),
         ])

    def test_extract_structure_missing(self):
        self.assertEqual(extract_structure(""), [])

    def test_rejects_higher_headings_later(self):
        """
        Ensures that if the first heading is e.g. h2, no h1 headings
        are allowed
        """
        self.assertRaises(IncorrectHeadings, extract_structure, "<h2>Hello</h2><h1>Hi</h1>")

    def test_rejects_improper_headings(self):
        self.assertRaises(IncorrectHeadings, extract_structure, "<h1>Hello</h1><h3>Bad heading</h3>")

    def test_use_existing_sect_ids(self):
        html = "<h1 id='h1_10'>Hi</h1><h1>There</h1>"
        structure = get_structure(parse(html))
        self.assertEqual(structure[0].sect_id, "h1_10")
        self.assertEqual(structure[1].sect_id, "h1_1")

    def test_dont_use_duplicate_existing_sect_id(self):
        html = "<h1 id='h1_10'>Hi</h1><h1 id='h1_10'>There</h1>"
        structure = get_structure(parse(html))
        self.assertEqual(structure[0].sect_id, "h1_10")
        self.assertEqual(structure[1].sect_id, "h1_1")

    def test_regression_1(self):
        # A bug in using existing section ids
        html = '<h1 id="h1_1">heading 1</h1><h1>A new heading</h1><h1 id="h1_2">heading 2</h1><h1 id="h1_3">heading 3</h1>'
        structure = get_structure(parse(html))
        self.assertEqual(["h1_1", "h1_4", "h1_2", "h1_3"], [s.sect_id for s in structure])

    def test_name_empty_headings(self):
        """
        Checks that we get some name for a heading with no text content
        """
        html = '<h1><img src="" /></h1>'
        structure = get_structure(parse(html))
        self.assertTrue(len(structure[0].name) > 0)

    def test_name_empty_para(self):
        """
        Checks that we get some name for a paragaph with no text content
        """
        html = '<p><img src="" /></p>'
        structure = get_structure(parse(html))
        self.assertTrue(len(structure[0].name) > 0)

    def test_name_empty_para(self):
        """
        Checks that we get some name for a paragaph with no text content
        """
        html = '<p><img src="" /></p>'
        structure = get_structure(parse(html))
        self.assertTrue(len(structure[0].name) > 0)

    def test_extract_empty(self):
        pres, out_html = extract_presentation('')
        self.assertEqual('', out_html)


class TestPresentationInfo(TestCase):
    def test_equality(self):
        p1 = PresentationInfo(prestype="command", name="foo", verbose_name="blah")
        p2 = PresentationInfo(prestype="command", name="foo")
        p3 = PresentationInfo(prestype="command", name="bar")
        self.assertEqual(p1, p2)
        self.assertNotEqual(p2, p3)
        self.assertEqual(set([p1]), set([p2]))


class TestFormat(TestCase):
    def setUp(self):
        # monkey patch to ensure some assumptions we make about LayoutDetails.
        # We may have to go the whole hog and do dependency injection at some
        # point.
        self._old_max_columns = LayoutDetails.max_columns
        LayoutDetails.max_columns = 4
        super(TestCase, self).setUp()

    def tearDown(self):
        LayoutDetails.max_columns = self._old_max_columns
        super(TestCase, self).tearDown()

    def test_remove_command_divs(self):
        self.assertEqual('<p>Test</p>', format_html('<div class="newrow">* </div><p>Test</p>', {}))
        self.assertEqual('<p>Test</p>', format_html('<p class="newrow">* </p><p>Test</p>', {}))

    def test_empty(self):
        self.assertEqual('', format_html('', {}));

    def test_no_headings(self):
        html = '<p>Test</p>'
        outh = '<p>Test</p>'
        self.assertEqual(outh, format_html(html, {}))

    def test_unknown_block_elements(self):
        """
        Ensure we don't remove block elements that we don't
        know about
        """
        html = '<foo>Test</foo>'
        outh = '<foo>Test</foo>'
        self.assertEqual(outh, format_html(html, {}))

    def test_existing_divs(self):
        html = "<div><foo><bar><fribble><div><div>Some text <p>para</p> some more</div><div> more <span> of </span> this stuff </div></div></fribble></bar></foo></div>"
        outh = '<p><foo><bar><fribble><p>Some text para some more more  of  this stuff </p></fribble></bar></foo></p>'
        self.assertEqual(outh, format_html(html, {}))

    def test_add_css_classes(self):
        html = "<h1>Hello <em>you</em></h1><h2>Hi</h2>"
        outh = '<h1 class=\"myclass\">Hello <em>you</em></h1><h2 class=\"c1 c2\">Hi</h2>'
        pres = {'h1_1':[PC('myclass')],
                'h2_1':[PC('c1'), PC('c2')]}
        self.assertEqual(outh, format_html(html, pres))

    def test_sanity_check_columns(self):
        """
        Check that user is not allowed to add column structure
        to blocks that aren't 'top level' in document structure
        """
        html = "<blockquote><p>How are you</p></blockquote>"
        pres = {'newrow_p_1': [NEWROW]}
        self.assertRaises(BadStructure, format_html, html, pres)

        html2 = "<blockquote><p>How are you</p></blockquote>"
        pres2 = {'newcol_p_1': [NEWROW]}
        self.assertRaises(BadStructure, format_html, html2, pres2)

    def test_check_command_order(self):
        """
        Check that the user will be prompted if they try to use,
        for example, a New Column command without a New row
        """
        html = "<h1>1</h1>"
        pres = {'newcol_h1_1':[NEWCOL]}
        self.assertRaises(BadStructure, format_html, html, pres)

    def test_columns_1(self):
        html = "<h1>1</h1><p>para 1</p><h1>2</h1><h1>3</h1>"
        outh = "<div class=\"row columns2\"><div class=\"column firstcolumn\"><div><h1>1</h1><p>para 1</p></div></div><div class=\"column lastcolumn\"><div><h1>2</h1><h1>3</h1></div></div></div>"
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_2':[NEWCOL]}
        self.assertEqual(outh, format_html(html, pres))

    def test_columns_with_double_width(self):
        html = "<h1>1</h1><p>para 1</p><h1>2</h1>"
        outh = "<div class=\"row columns3\"><div class=\"column firstcolumn doublewidth\"><div><h1>1</h1><p>para 1</p></div></div><div class=\"column lastcolumn\"><div><h1>2</h1></div></div></div>"
        pres =  {'newrow_h1_1':[NEWROW],
                 'newcol_h1_1':[NEWCOL, PC('doublewidth', column_equiv=2)],
                 'newcol_h1_2':[NEWCOL]}
        self.assertEqual(outh, format_html(html, pres))

    def test_columns_with_double_width_2(self):
        html = "<h1>1</h1><p>para 1</p><h1>2</h1>"
        outh = "<div class=\"row columns3\"><div class=\"column firstcolumn\"><div><h1>1</h1><p>para 1</p></div></div><div class=\"column lastcolumn doublewidth\"><div><h1>2</h1></div></div></div>"
        pres =  {'newrow_h1_1':[NEWROW],
                 'newcol_h1_1':[NEWCOL],
                 'newcol_h1_2':[NEWCOL, PC('doublewidth', column_equiv=2)]}
        self.assertEqual(outh, format_html(html, pres))

    def test_max_cols(self):
        """
        Check we can't exceed max cols
        """
        html = "<h1>1</h1><h1>2</h1><h1>3</h1><h1>4</h1><h1>5</h1>"
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_2':[NEWCOL],
                'newcol_h1_3':[NEWCOL],
                'newcol_h1_4':[NEWCOL],
                'newcol_h1_5':[NEWCOL]
                }
        self.assertRaises(TooManyColumns, format_html, html, pres)
    def test_max_cols_2(self):
        """
        Check we can't exceed max cols with double width cols
        """
        html = "<h1>1</h1><h1>2</h1><h1>3</h1>"
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_1':[NEWCOL, PC('doublewidth', column_equiv=2)],
                'newcol_h1_2':[NEWCOL, PC('doublewidth', column_equiv=2)],
                'newcol_h1_3':[NEWCOL],
                }
        self.assertRaises(TooManyColumns, format_html, html, pres)


    def test_columns_2(self):
        html = ("<h1>1</h1>"
                "<h1>2</h1>"
                "<h2>2.1</h2>"
                "<h2>2.2</h2>"
                "<h2>2.3</h2>"
                "<h2>2.4</h2>"
                "<h1>3</h1>"
                "<h1>4</h1>")

        pres = {'newrow_h2_1':[NEWROW],
                'newcol_h2_2':[NEWCOL],
                'newrow_h2_3':[NEWROW],
                'newcol_h2_4':[NEWCOL],
                'newrow_h1_3':[NEWROW],
                'newcol_h1_4':[NEWCOL],
                }

        outh = ("<h1>1</h1>"
                "<h1>2</h1>"
                "<div class=\"row columns2\">"
                  "<div class=\"column firstcolumn\">"
                    "<div>"
                      "<h2>2.1</h2>"
                    "</div>"
                  "</div>"
                  "<div class=\"column lastcolumn\">"
                    "<div>"
                      "<h2>2.2</h2>"
                    "</div>"
                  "</div>"
                "</div>"
                "<div class=\"row columns2\">"
                  "<div class=\"column firstcolumn\">"
                    "<div>"
                      "<h2>2.3</h2>"
                    "</div>"
                  "</div>"
                  "<div class=\"column lastcolumn\">"
                    "<div>"
                      "<h2>2.4</h2>"
                    "</div>"
                  "</div>"
                "</div>"
                "<div class=\"row columns2\">"
                  "<div class=\"column firstcolumn\">"
                    "<div>"
                      "<h1>3</h1>"
                    "</div>"
                  "</div>"
                  "<div class=\"column lastcolumn\">"
                    "<div>"
                      "<h1>4</h1>"
                    "</div>"
                  "</div>"
                "</div>")
        self.assertEqual(outh, format_html(html, pres))

    def test_layout_with_styling(self):
        html = "<h1>1</h1><p>para 1</p><h1>2</h1><h1>3</h1>"
        outh = "<div class=\"row columns2 fancyrow\"><div class=\"column firstcolumn\"><div><h1>1</h1><p>para 1</p></div></div><div class=\"column lastcolumn\"><div class=\"fancycol\"><h1>2</h1><h1>3</h1></div></div></div>"
        pres = {'newrow_h1_1':[NEWROW, PC('fancyrow')],
                'newcol_h1_2':[NEWCOL, PC('fancycol')]}
        self.assertEqual(outh, format_html(html, pres))

    def test_columns_single_col(self):
        html = "<h1>1</h1><p>para 1</p><h2>2</h2>"
        outh = "<div class=\"row\"><div><div><h1>1</h1><p>para 1</p><h2>2</h2></div></div></div>"
        pres =  {'newrow_h1_1':[NEWROW]}
        self.assertEqual(outh, format_html(html, pres))

    def test_nested_layout(self):
        html = ("<h1>1</h1>"
                "<h1>2</h1>"
                "<h1>3</h1>"
                "<h1>4</h1>"
                "<h1>5</h1>"
                "<h1>6</h1>"
                "<h1>7</h1>")
        # - row 1:
        #   - 3 columns
        #   - middle column has inner row structure with
        #     - 1 row
        #     - 2 columns
        # - row 2:
        #   - 2 columns
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_2':[NEWCOL],
                'innerrow_h1_3':[NEWINNERROW],
                'innercol_h1_4':[NEWINNERCOL],
                'newcol_h1_5':[NEWCOL],
                'newrow_h1_6':[NEWROW],
                'newcol_h1_7':[NEWCOL],
                }
        outh = ('<div class="row columns3">'
                  '<div class="column firstcolumn">'
                    '<div><h1>1</h1></div>'
                  '</div>'
                  '<div class="column">'
                    '<div>'
                      '<h1>2</h1>'
                      '<div class="row columns2">'
                        '<div class="column firstcolumn">'
                          '<div><h1>3</h1></div>'
                        '</div>'
                        '<div class="column lastcolumn">'
                          '<div><h1>4</h1></div>'
                        '</div>'
                      '</div>'
                    '</div>'
                  '</div>'
                  '<div class="column lastcolumn">'
                      '<div><h1>5</h1></div>'
                  '</div>'
                '</div>'
                '<div class="row columns2">'
                  '<div class="column firstcolumn">'
                    '<div><h1>6</h1></div>'
                  '</div>'
                  '<div class="column lastcolumn">'
                    '<div><h1>7</h1></div>'
                  '</div>'
                '</div>'
                )
        self.assertEqual(outh, format_html(html, pres))

    def test_nested_layout_2(self):
        html = ("<h1>1</h1>"
                "<h1>2</h1>"
                "<h1>3</h1>"
                "<h1>4</h1>")
        # - row 1:
        #   - 2 columns
        #   - 2nd middle column has inner row structure with
        #     - 1 row
        #     - 2 columns
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_2':[NEWCOL],
                'innerrow_h1_3':[NEWINNERROW],
                'innercol_h1_4':[NEWINNERCOL],
                }
        outh = ('<div class="row columns2">'
                  '<div class="column firstcolumn">'
                    '<div><h1>1</h1></div>'
                  '</div>'
                  '<div class="column lastcolumn">'
                    '<div>'
                      '<h1>2</h1>'
                      '<div class="row columns2">'
                        '<div class="column firstcolumn">'
                          '<div><h1>3</h1></div>'
                        '</div>'
                        '<div class="column lastcolumn">'
                          '<div><h1>4</h1></div>'
                        '</div>'
                      '</div>'
                    '</div>'
                  '</div>'
                '</div>'
                )
        self.assertEqual(outh, format_html(html, pres))

    def test_max_cols_nested(self):
        """
        Tests the error message for when the first col contains a nested layout.
        """
        html = "<h1>1</h1><h1>2</h1><h1>3</h1><h1>4</h1><h1>5</h1><h1>6</h1>"
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_1':[NEWCOL],
                'innerrow_h1_1':[NEWINNERROW],
                'innercol_h1_2':[NEWINNERCOL],
                'newcol_h1_3':[NEWCOL],
                'newcol_h1_4':[NEWCOL],
                'newcol_h1_5':[NEWCOL],
                'newcol_h1_6':[NEWCOL],
                }
        self.assertRaises(TooManyColumns, format_html, html, pres)

    def test_max_cols_nested_2(self):
        """
        Tests TooManyColumns is raised when the nested layout has too many columns
        """
        html = "<h1>1</h1><h1>2</h1><h1>3</h1><h1>4</h1><h1>5</h1><h1>6</h1>"
        pres = {'newrow_h1_1':[NEWROW],
                'newcol_h1_1':[NEWCOL],
                'innerrow_h1_1':[NEWINNERROW],
                'innercol_h1_2':[NEWINNERCOL],
                'innercol_h1_3':[NEWINNERCOL],
                'innercol_h1_4':[NEWINNERCOL],
                'innercol_h1_5':[NEWINNERCOL],
                'newcol_h1_6':[NEWCOL],
                }
        self.assertRaises(TooManyColumns, format_html, html, pres)

    def test_format_pre(self):
        html = "<pre>This\r\nis\r\na\r\ntest</pre>"
        # check that format_html doesn't do anything nasty inside the pre
        html2 = format_html(html, {})
        pres, html3 = extract_presentation(html2)
        self.assertEqual(html, html3)

    def test_pretty_print(self):
        # Pretty print is used by the view function, so that the output HTML is
        # easier to read by web designers who may need to debug it
        html = "<ul><li>1</li></ul>"
        # The exact HTML currently looks like this:
        pretty_html = """
  
    <ul>
      <li>1</li>
    </ul>
  

"""
        out_html = format_html(html, {}, pretty_print=True)
        self.assertEqual(pretty_html, out_html)

    def test_round_trip(self):
        # Test the round trip of html -> extract_presentation -> format_html
        stored_html = "<h1>Hello</h1>"
        pres, simple_html = extract_presentation(stored_html)
        formatted = format_html(simple_html, pres)
        self.assertEqual(formatted, stored_html)


class TestPreview(TestCase):

    def test_preview(self):
        html = "<h1>Hello <b>there</b></h1><p><ul><li>An item</li></ul></p>"
        expected = ('<div class="structural tagh1">Hello there</div>'
                    '<div class="structural tagul">An item...</div>')
        formatted = preview_html(html, {})
        self.assertEqual(expected, formatted)

class TestHacks(TestCase):
    def test_div_format_hack(self):
        """
        Check that we can convert 'p' tags into 'div' using the 'div' class hack
        """
        html = '<p>Test</p>'
        pres = {'p_1':[PC('div')]}
        outh = '<div class="div">Test</div>'
        self.assertEqual(outh, format_html(html, pres))

    def test_div_extract_hack(self):
        """
        Check that a div with class "div" is recognised and turned back into a 'p'
        when extracting
        """
        html = '<div class="div">Test</div>'
        pres, html2 = extract_presentation(html)
        self.assertEqual({'p_1':set([PC('div')])}, pres)
        self.assertEqual('<p id="p_1">Test</p>', html2);

    def test_plugin_p_hack(self):
        """
        Check that a 'p' with only a plugin object is converted to a 'div'
        """
        # NB: current implementation of plugin objects is that they are
        # represented by an image in the editor.  Our code has to run before
        # these are converted, so we have to work with this implementation detail.
        html = '<p> <img src="blah" id="plugin_obj_123"/></p>'
        outh = '<div class="div"> <img src="blah" id="plugin_obj_123"/></div>'
        self.assertEqual(outh, format_html(html, {}))

    def test_plugin_p_hack_empty_only(self):
        """
        Check that if 'p' has any text in it, it is not converted
        """
        html = '<p>X <img src="blah" id="plugin_obj_123" /></p>'
        outh = '<p>X <img src="blah" id="plugin_obj_123"/></p>'
        self.assertEqual(outh, format_html(html, {}))
        html2 = '<p> <img src="blah" id="plugin_obj_123" />X</p>'
        outh2 = '<p> <img src="blah" id="plugin_obj_123"/>X</p>'
        self.assertEqual(outh2, format_html(html2, {}))


class TestElementTreeUtils(TestCase):
    def test_get_parent(self):
        """
        Tests that get_parent works
        """
        t = parse("<a><b1></b1><b2></b2></a>")
        n = t.find(".//b2")
        p = get_parent(t, n)
        self.assertEqual(p, t.find(".//a"))

    def test_get_index(self):
        """
        Tests that get_index returns the index of node amongst its siblings
        """
        t = parse("<a><b1></b1><b2></b2></a>")
        n = t.find(".//b2")
        p = get_parent(t, n)
        self.assertEqual(1, get_index(p, n))

    def test_eliminate_tag_1(self):
        t = ET.fromstring("<a>Hello<b>Goodbye</b>End</a>")
        eliminate_tag(t, 0)
        self.assertEqual("<a>HelloGoodbyeEnd</a>", ET.tostring(t))

    def test_eliminate_tag_2(self):
        t = ET.fromstring("<a>Hello<b>Goodbye</b>Some<b>More</b>End</a>")
        eliminate_tag(t, 0)
        self.assertEqual("<a>HelloGoodbyeSome<b>More</b>End</a>", ET.tostring(t))

    def test_eliminate_tag_3(self):
        t = ET.fromstring("<a>Hello<b>Goodbye</b>Some<b>More</b>End</a>")
        eliminate_tag(t, 1)
        self.assertEqual("<a>Hello<b>Goodbye</b>SomeMoreEnd</a>", ET.tostring(t))

    def test_eliminate_tag_4(self):
        t = ET.fromstring("<a>Hello<b>Good<x>b</x><y>y</y>e</b>End</a>")
        eliminate_tag(t, 0)
        self.assertEqual("<a>HelloGood<x>b</x><y>y</y>eEnd</a>", ET.tostring(t))

    def test_eliminate_tag_5(self):
        t = ET.fromstring("<a>Hello<b>First <c>node</c></b>tail<b>Good<x>b</x><y>y</y>e</b>And<b>Stuff</b></a>")
        eliminate_tag(t, 1)
        self.assertEqual("<a>Hello<b>First <c>node</c></b>tailGood<x>b</x><y>y</y>eAnd<b>Stuff</b></a>", ET.tostring(t))



class TestExtractPresentation(TestCase):
    maxDiff = None
    def test_extract_presentation(self):
        html = "<h1 class=\"foo\">Heading 1</h1><h2 class=\"bar baz\">Heading 2</h2><p class=\"whatsit\">Some paragraph</p>"
        pres, html2 = extract_presentation(html)
        self.assertEqual({'h1_1':set([PC('foo')]),
                          'h2_1':set([PC('bar'), PC('baz')]),
                          'p_1':set([PC('whatsit')]),
                          }, pres)
        self.assertEqual("<h1 id=\"h1_1\">Heading 1</h1><h2 id=\"h2_1\">Heading 2</h2><p id=\"p_1\">Some paragraph</p>", html2)

    # Lazy method - assume that combine works and check the round-trip.
    # This only works currently if we 'normalise' the presentation dict.
    def test_extract_1(self):
        html = \
            "<h1>1</h1>" \
            "<h1>2</h1>" \
            "<h2>2.1</h2>" \
            "<h2>2.2</h2>" \
            "<h2>2.3</h2>" \
            "<h2>2.4</h2>" \
            "<h1>3</h1>" \
            "<h1>4</h1>"

        presentation = {'h1_1':set([PC('myclass1')]),
                        'h1_2':set([]),
                        'h1_3':set([]),
                        'h1_4':set([]),
                        'h2_1':set([]),
                        'h2_2':set([]),
                        'h2_3':set([]),
                        'h2_4':set([]),
                        'newrow_h1_1':set([NEWROW]),
                        'newcol_h1_1':set([NEWCOL]),
                        'newrow_h2_1':set([NEWROW]),
                        'newcol_h2_1':set([NEWCOL]),
                        'newcol_h2_2':set([NEWCOL]),
                        'newrow_h2_3':set([NEWROW]),
                        'newcol_h2_3':set([NEWCOL]),
                        'newcol_h2_4':set([NEWCOL, PC('myclass2')]),
                        'newrow_h1_3':set([NEWROW]),
                        'newcol_h1_3':set([NEWCOL]),
                        'newcol_h1_4':set([NEWCOL]),
                        }
        combined = format_html(html, presentation)
        pres2, html2 = extract_presentation(combined)
        self.assertEqual(presentation, pres2)

    def test_extract_2(self):
        # Full featured, proper test
        html = """
<div class="row columns3"><div class="column firstcolumn"><div class="myclass"><h1>Hello Jane</h1><p>Some fancy content, entered using WYMeditor</p><p>Another paragraph</p><p>Hello</p></div></div><div class="column doublewidth"><div><h1>Another &lt;heading&gt;</h1><h2>this is a test</h2><h2>hello1</h2><h3>hello2</h3><h3>hello3</h3><h3>hello4</h3></div></div><div class="column lastcolumn"><div><h1>hello5</h1><h2>hello6</h2><p>asdasd</p><p>asdxx</p></div></div></div>
"""
        pres = {'newrow_h1_1':set([NEWROW]),
                'newcol_h1_1':set([NEWCOL, PC('myclass')]),
                'h1_1':set(),
                'newcol_h1_2':set([NEWCOL, PC('doublewidth', column_equiv=2)]),
                'h1_2':set(),
                'p_1': set(),
                'p_2': set(),
                'p_3': set(),
                'h2_1':set(),
                'h2_2':set(),
                'h3_1':set(),
                'h3_2':set(),
                'h3_3':set(),
                'newcol_h1_3':set([NEWCOL]),
                'h1_3':set(),
                'h2_3':set(),
                'p_4': set(),
                'p_5': set(),
                }

        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)

    def test_extract_3(self):
        # Tests some other boundary conditions e.g. 1 column row,
        html = """
<div class="row"><div><div><h1>1</h1><h2>1.1</h2><h2>1.2</h2></div></div></div>
"""
        pres = {'h1_1': set(),
                'newrow_h1_1':set([NEWROW]),
                'newcol_h1_1':set([NEWCOL]),
                'h2_1': set(),
                'h2_2': set(),
                }
        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)

    def test_extract_no_inner_col_div_1(self):
        # Tests that we can extract column structure if we don't have
        # an inner column div.
        # This is important for the case where LayoutDetails.use_inner_column_div = False

        # Single col structure
        html = """
<div class="row"><div><h1>1</h1><h2>1.1</h2><h2>1.2</h2></div></div>
"""
        pres = {'h1_1': set(),
                'newrow_h1_1':set([NEWROW]),
                'newcol_h1_1':set([NEWCOL]),
                'h2_1': set(),
                'h2_2': set(),
                }
        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)

    def test_extract_no_inner_col_div_2(self):
        # Tests that we can extract column structure if we don't have 
        # an inner column div.
        # This is important for the case where LayoutDetails.use_inner_column_div = False

        # Double col structure
        html = """
<div class="row"><div class="column firstcolumn"><h1>1</h1></div><div class="column lastcolumn"><h2>1.1</h2></div></div>
"""
        pres = {'h1_1': set(),
                'newrow_h1_1':set([NEWROW]),
                'newcol_h1_1':set([NEWCOL]),
                'h2_1': set(),
                'newcol_h2_1':set([NEWCOL])
                }
        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)

    def test_extract_nested_layout(self):
        """
        Tests that we can properly extract a layout created using inner rows/columns.
        """
        pres = {'newrow_h1_1':set([NEWROW]),
                'newcol_h1_1':set([NEWCOL]),
                'newcol_h1_2':set([NEWCOL]),
                'innerrow_h1_3':set([NEWINNERROW]),
                'innercol_h1_3':set([NEWINNERCOL]),
                'innercol_h1_4':set([NEWINNERCOL]),
                'newcol_h1_5':set([NEWCOL]),
                'newrow_h1_6':set([NEWROW]),
                'newcol_h1_6':set([NEWCOL]),
                'newcol_h1_7':set([NEWCOL]),
                'h1_1':set([]),
                'h1_2':set([]),
                'h1_3':set([]),
                'h1_4':set([]),
                'h1_5':set([]),
                'h1_6':set([]),
                'h1_7':set([]),
                }
        html = ('<div class="row columns3">'
                  '<div class="column firstcolumn">'
                    '<div><h1>1</h1></div>'
                  '</div>'
                  '<div class="column">'
                    '<div>'
                      '<h1>2</h1>'
                      '<div class="row columns2">'
                        '<div class="column firstcolumn">'
                          '<div><h1>3</h1></div>'
                        '</div>'
                        '<div class="column lastcolumn">'
                          '<div><h1>4</h1></div>'
                        '</div>'
                      '</div>'
                    '</div>'
                  '</div>'
                  '<div class="column lastcolumn">'
                      '<div><h1>5</h1></div>'
                  '</div>'
                '</div>'
                '<div class="row columns2">'
                  '<div class="column firstcolumn">'
                    '<div><h1>6</h1></div>'
                  '</div>'
                  '<div class="column lastcolumn">'
                    '<div><h1>7</h1></div>'
                  '</div>'
                '</div>'
                )
        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)


    def test_extract_nested_layout_2(self):
        """
        Issue 37
        """
        pres = {'newrow_p_1':set([NEWROW]),
                'newcol_p_1':set([NEWCOL]),
                'innerrow_p_1':set([NEWINNERROW]),
                'innercol_p_1':set([NEWINNERCOL]),
                'innercol_p_2':set([NEWINNERCOL]),
                'newcol_p_3':set([NEWCOL]),
                'innerrow_p_3':set([NEWINNERROW]),
                'innercol_p_3':set([NEWINNERCOL]),
                'innercol_p_4':set([NEWINNERCOL]),
                'p_1':set([]),
                'p_2':set([]),
                'p_3':set([]),
                'p_4':set([]),
                }
        html = ('<div class="row columns2">'
                  '<div class="column firstcolumn">'
                    '<div>'
                      '<div class="row columns2">'
                        '<div class="column firstcolumn">'
                          '<div><p>col1</p></div>'
                        '</div>'
                        '<div class="column lastcolumn">'
                          '<div><p>col2</p></div>'
                        '</div>'
                      '</div>'
                    '</div>'
                  '</div>'
                  '<div class="column firstcolumn">'
                    '<div>'
                      '<div class="row columns2">'
                        '<div class="column firstcolumn">'
                          '<div><p>col3</p></div>'
                        '</div>'
                        '<div class="column lastcolumn">'
                          '<div><p>col4</p></div>'
                        '</div>'
                      '</div>'
                    '</div>'
                  '</div>'
                '</div>'
                )
        pres2, html2 = extract_presentation(html)
        self.assertEqual(pres, pres2)


class TestHtmlCleanup(TestCase):
    safari_example_1 = """
<p style="margin-top: 0px; margin-right: 0px; margin-bottom: 0.8em; margin-left: 0px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; font-size: 0.9em; line-height: 1.4em; "><strong style="font-weight: bold; ">Formerly: Community Health Sciences Research (CHSR) IRG</strong></p><p style="margin-top: 0px; margin-right: 0px; margin-bottom: 0.8em; margin-left: 0px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; font-size: 0.9em; line-height: 1.4em; ">The Clinical Epidemiology IRG aims to undertake research that makes an important difference to patient care. Our work is divided into two broad research areas:</p><h4 style="color: rgb(153, 0, 51); margin-top: 0px; margin-right: 0px; margin-bottom: 0.25em; margin-left: 0px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; font-size: 1.1em; line-height: 1.3em; "><strong style="font-weight: bold; ">Clinical and environmental epidemiology -</strong>&#160;including</h4><ul style="margin-top: 0px; margin-right: 0px; margin-bottom: 1.5em; margin-left: 0px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; line-height: 1.4em; font-size: 0.9em; "><li style="margin-top: 0px; margin-right: 0px;margin-bottom: 0.25em; margin-left: 20px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px;padding-left: 0px; ">mental health</li><li style="margin-top: 0px; margin-right: 0px; margin-bottom: 0.25em; margin-left: 20px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; ">child protection</li><li style="margin-top: 0px; margin-right: 0px; margin-bottom: 0.25em; margin-left: 20px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px;">cancer</li><li style="margin-top: 0px; margin-right: 0px; margin-bottom: 0.25em; margin-left: 20px; padding-top: 0px; padding-right: 0px; padding-bottom: 0px; padding-left: 0px; ">environmental, economic and social risk factors</li></ul></span>
"""
    safari_output_1 = """
<p><strong>Formerly: Community Health Sciences Research (CHSR) IRG</strong></p><p>The Clinical Epidemiology IRG aims to undertake research that makes an important difference to patient care. Our work is divided into two broad research areas:</p><h4><strong>Clinical and environmental epidemiology -</strong> including</h4><ul><li>mental health</li><li>child protection</li><li>cancer</li><li>environmental, economic and social risk factors</li></ul>"""

    firefox_oowriter_example_1 = u"""
<style type="text/css">
	&lt;!--
		@page { margin: 2cm }
		P { margin-bottom: 0.21cm }
		H2 { margin-bottom: 0.21cm }
		H2.western { font-family: "Bitstream Vera Sans", sans-serif; font-size: 14pt; font-style: italic }
		H2.cjk { font-family: "DejaVu Sans"; font-size: 14pt; font-style: italic }
		H2.ctl { font-family: "DejaVu Sans"; font-size: 14pt; font-style: italic }
	--&gt;
	</style><p class="western"><strong>My Café</strong></p><h2 class="western">Heading</h2><table width="459" cellpadding="4"><col width="110"><col width="334"><tbody><tr><td><p class="western">cell1</p></td><td><p class="western">cell2</p></td></tr></tbody><p class="western"></p><h2 class="western">Heading 2</h2><p class="western"></p><p class="western">Some “text”</p></col>
"""
    firefox_oowriter_output_1 = u"""
<p><strong>My Caf&#233;</strong></p><h2>Heading</h2><p>cell1</p><p>cell2</p><h2>Heading 2</h2><p>Some “text”</p>
"""

    def assertEqualClean(self, input, output):
        """
        Assert that expected output is the same as the input cleaned
        """
        # Do a pretty_print to make error messages nicer
        actual_output = clean_html(input)
        s1 = pretty_print(output).strip()
        s2 = pretty_print(actual_output).strip()
        try:
            self.assertEqual(s1, s2)
        except:
            print
            print s1
            print
            print s2
            raise

    def test_cleanup_safari_1(self):
        self.assertEqualClean(self.safari_example_1,
                              self.safari_output_1)

    def test_cleanup_firefox_oowriter_1(self):
        self.assertEqualClean(self.firefox_oowriter_example_1,
                              self.firefox_oowriter_output_1)

    def test_cleanup_tables(self):
        self.assertEqualClean("<table><tbody><tr><td><p>Hello</p></td></tr></tbody><p>P2</p>text</table>",
                              "<p>Hello</p><p>P2</p><p>text</p>");

    def test_toplevel_text(self):
        # Make sure that text at the top level is inside some tag
        self.assertEqualClean("test", "<p>test</p>")

    def test_div_to_p(self):
        self.assertEqualClean("<div>Foo</div>", "<p>Foo</p>")

    def test_nested_p(self):
        self.assertEqualClean("<p>Hello <p>How are <p>you</p></p> today</p>",
                              "<p>Hello </p><p>How are </p><p>you</p><p> today</p>")

    def test_br_to_p(self):
        self.assertEqualClean("This is<br /><br />a test",
                              "<p>This is</p><p>a test</p>")

    def test_br_in_p(self):
        self.assertEqualClean("<p>line 1<br />line 2<br />line 3</p>",
                              "<p>line 1<br />line 2<br />line 3</p>")

    def test_p_in_li(self):
        self.assertEqualClean("<ul><li><p>An item</p></li></ul>",
                              "<ul><li>An item</li></ul>")

    def test_remove_span(self):
        self.assertEqualClean("<p>Some <span>text</span> with a silly span</p>",
                              "<p>Some text with a silly span</p>")

    def test_inline_wrapping_block(self):
        self.assertEqualClean("<strong><p>A test</p></strong>",
                              "<p>A test</p>")
        self.assertEqualClean("<em><ul><li>A test</li></ul></strong>",
                              "<ul><li>A test</li></ul>")

    def test_duplicate_ids(self):
        self.assertEqualClean('<p id="p_1">test</p><p id="p_1">test 2</p>' +
                              '<ul id="ul_1">test</ul><ul id="ul_1">test 2</p>',
                              '<p id="p_1">test</p><p>test 2</p>' +
                              '<ul id="ul_1">test</ul><ul>test 2</ul>')

    def test_remove_hard_spaces(self):
        # &nbsp;
        self.assertEqualClean('<p>&nbsp;Hello&nbsp;</p>',
                              '<p> Hello </p>')
        # &#160;
        self.assertEqualClean('<p>&#160;Hello&#160;</p>',
                              '<p> Hello </p>')

        # Some unicode
        self.assertEqualClean(u'<p>&nbsp;Frappé&nbsp;</p>',
                              u'<p> Frapp&#233; </p>')


class TestRetrieveStyles(TestCase):
    fixtures = ['test_classes.json']

    def test_retrieve_styles_all(self):
        classes = CssClass.objects.all()
        classes_2 = get_classes('cms_harness/example.html')
        self.assertEqual(len(classes), len(classes_2))

    def test_retrieve_styles_order(self):
        classes = get_classes('cms_harness/example.html')
        # Should be ordered by category, alphabetically, with classes with no
        # category at the beginning.
        categories = [c.category.name if c.category is not None else '' for c in classes]
        self.assertEqual(categories, sorted(categories))

        # Should be ordered alphabetically within categories
        c2 = [c for c in classes if c.category is not None and c.category.name == 'Borders']
        assert len(c2) > 0 # Sanity
        self.assertEqual(c2, list(CssClass.objects.filter(category__name='Borders').order_by('verbose_name')))



"""
Combine simple HTML and presentation info into rendered HTML
"""
from lxml import etree as ET

from semanticeditor.common import strip_presentation, get_classes_from_presinfo, html_extract, parse, get_structure
from semanticeditor.definitions import PREVIEW_BLOCKDEF, BLOCKDEF
from semanticeditor.layout import create_layout, check_layout, get_layout_details_strategy
from semanticeditor.utils.etree import indent

## Main functions and sub functions
def format_html(html, styleinfo, return_tree=False, pretty_print=False):
    """
    Formats the XHTML given using a dictionary of style information.
    The dictionary has keys which are the ids of sections,
    and values which are lists of CSS classes or special commands.
    """
    layout_strategy = get_layout_details_strategy()
    html = layout_strategy.format_pre_parse_hacks(html, styleinfo)
    root = parse(html, clean=True)
    root = layout_strategy.format_post_parse_hacks(root, styleinfo)
    structure = get_structure(root, assert_structure=True)
    structure = layout_strategy.format_structure_hacks(structure, styleinfo)
    sect_ids = [s.sect_id for s in structure]
    styleinfo = _sanitise_styleinfo(styleinfo, sect_ids)

    # Strip existing divs, otherwise we cannot format properly.  If
    # there are other block level elements that mess things up, we
    # raise BadStructure later, but divs have no semantics so can just
    # be removed.
    strip_presentation(root)

    # Apply normal CSS classes.
    for si in structure:
        # Apply css styles
        classes = get_classes_from_presinfo(styleinfo[si.sect_id])
        classes.sort()
        if classes:
            si.node.set("class", " ".join(classes))

    # Create layout from row/column commands
    layout = create_layout(root, styleinfo, structure)
    for c in layout.content:
        check_layout(c, structure, layout_strategy)
    # Create new ET tree from layout.  The individual nodes that belong to
    # 'root' are not altered, but just added to a new tree.  This means that the
    # information in 'structure' does not need updating.
    nodes = []

    for content in layout.content:
        nodes.extend(content.as_nodes(layout_strategy))
    rendered = ET.fromstring("<html><body></body></html>")
    rendered.getchildren()[0].extend(nodes)

    # Apply hacks
    rendered = layout_strategy.format_post_layout_hacks(rendered, structure, styleinfo)

    # Pretty print
    if pretty_print:
        indent(rendered)

    # Remove the temporary IDs we may have added when splitting the HTML
    # into content and presentation.  We don't do this before this point,
    # as the IDs need to be there to identify sections
    for si in structure:
        if 'id' in si.node.attrib:
            del si.node.attrib['id']

    if return_tree:
        return (rendered, structure)
    else:
        return html_extract(rendered)


def preview_html(html, pres):
    root, structure = format_html(html, pres, return_tree=True)
    structure2 = [si for si in structure if si.tag in PREVIEW_BLOCKDEF]
    known_nodes = dict((si.node, si) for si in structure2)
    _create_preview(root, structure2, known_nodes)
    return html_extract(root)

def _create_preview(node, structure, known_nodes):
    children = node.getchildren()
    if children and children[0].tag == 'body':
        children = children[0].getchildren()
    for n in children:
        if n.tag == 'div' and n not in known_nodes:
            _create_preview(n, structure, known_nodes)
        else:
            sect = known_nodes.get(n)
            if sect is not None and (n.tag in BLOCKDEF or n.tag == 'div'):
                n.set('class', 'structural ' + "tag" + n.tag.lower())
                n.tag = "div"
                n[:] = []
                n.text = sect.name
            else:
                node.remove(n)


def _sanitise_styleinfo(styleinfo, sect_ids):
    # Replace lists with sets
    out = {}
    for k, v in styleinfo.items():
        out[k] = set(v)

    # Ensure that all sections have an entry in styleinfo
    for sect_id in sect_ids:
        if not sect_id in out:
            out[sect_id] = set()

    return out



"""
Definitions used for HTML manipulation
"""

### Errors ###

class IncorrectHeadings(ValueError):
    pass

class BadStructure(ValueError):
    pass

class TooManyColumns(BadStructure):
    pass

AllUserErrors = (IncorrectHeadings, BadStructure, TooManyColumns)


### Definitions ###

_TECHNICAL_BLOCKDEF = set(['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'ol', 'ul', 'blockquote']) # according to HTML4
_ADDITIONAL_BLOCKDEF = set(['li']) # li really act like block elements
BLOCKDEF = _TECHNICAL_BLOCKDEF | _ADDITIONAL_BLOCKDEF
BLOCKDEF_SELECTOR = ",".join(BLOCKDEF) # need to sync with wymeditor.semantic.js
HEADINGDEF = set(['h1', 'h2', 'h3', 'h4', 'h5', 'h6'])
PREVIEW_BLOCKDEF = _TECHNICAL_BLOCKDEF

# The number of chars we trim block level elements to.
BLOCK_LEVEL_TRIM_LENGTH = 200

### Presentation information ###

class PresentationInfo(object):
    """
    Encapsulates a piece of presentation information.
    """
    def __init__(self, prestype=None, name=None, verbose_name="", description="", allowed_elements=None, column_equiv=None):
        self.prestype = prestype
        self.name = name
        # verbose_name, description and allowed_elements are additional pieces
        # of information that are only needed when the client is requesting a
        # list of styles.  In other situations these objects may not have these
        # attributes filled in.
        self.verbose_name = verbose_name
        self.description = description
        if allowed_elements is None:
            allowed_elements = []
        self.allowed_elements = allowed_elements
        self.column_equiv = column_equiv

    def __eq__(self, other):
        return self.prestype == other.prestype and self.name == other.name

    def __hash__(self):
        return hash(self.prestype) ^ hash(self.name)

    def __repr__(self):
        return "PresentationInfo(prestype=\"%s\", name=\"%s\")" % (self.prestype, self.name)


class PresentationClass(PresentationInfo):
    """
    A PresentationInfo representing a CSS class applied to a section.
    """
    def __init__(self, name, verbose_name="", description="",
                 allowed_elements=None, column_equiv=None,
                 category=None):
        super(PresentationClass, self).__init__(prestype="class",
                                                name=name,
                                                verbose_name=verbose_name,
                                                description=description,
                                                allowed_elements=allowed_elements,
                                                column_equiv=column_equiv)
        self.category = category


class PresentationCommand(PresentationInfo):
    """
    A PresentationInfo representing a command applied to a section
    """
    def __init__(self, name, layout_order, verbose_name="", description=""):
        super(PresentationCommand, self).__init__(prestype="command",
                                                  name=name,
                                                  verbose_name=verbose_name,
                                                  description=description,
                                                  allowed_elements=sorted(list(_TECHNICAL_BLOCKDEF)))
        # The order that the command must appear in the document e.g. NEWROW
        # appears before NEWCOL.  (If there isn't a simple ordering on these
        # commands, the logic in _create_layout may need to be re-thought)
        self.layout_order = layout_order


    @property
    def prefix(self):
        """
        This is a prefix used to generate a name used for registering this
        command against a section and storing other presentation info.
        """
        # If there is, for example, a section 'h1_1' in the document, this
        # prefix is used to generate e.g. newrow_h1_1. The presence of the
        # name 'newrow_h1_1' says that a 'newrow' command was used against
        # h1_1, and it also allows PresentationClass objects to be stored
        # against newrow_h1_1 itself, which represents the entire row.

        return self.name + "_"


### DEFINITONS OF COMMANDS ###

NEWROW = PresentationCommand('newrow',
                             0,
                             verbose_name="new row",
                             description="""
<p>Use this command to start a new row.</p>

<p>This must be used in conjunction with 'New column'
to create a column layout.</p>

<p>If you wish to stop an existing column layout for a section, then you will
need to apply a 'New row' command to that section, creating a row with
just one column in it.</p>

""")

NEWCOL = PresentationCommand('newcol',
                             1,
                             verbose_name="new column",
                             description="""
<p>Use this command to start a new column, after a 'New row'
command has been used to start a set of columns.</p>

""")

NEWINNERROW = PresentationCommand('innerrow',
                                  2,
                                  verbose_name="inner row",
                                  description="""
<p>Use this command to start a nested, inner row within an existing column
structure.</p>

<p>This must be used in conjunction with 'Inner column'
to create a column layout.</p>

""")


NEWINNERCOL = PresentationCommand('innercol',
                                  3,
                                  verbose_name="inner column",
                                  description="""
<p>Use this command to start a new inner column, after an 'Inner row' command
has been used to start a set of nested columns.</p>

""")

COMMANDS = [NEWROW, NEWCOL, NEWINNERROW, NEWINNERCOL]

SORTED_COMMANDS = sorted(COMMANDS, key=lambda c: c.layout_order)

for i, c in enumerate(SORTED_COMMANDS):
    # Several places that index SORTED_COMMANDS make this assumption:
    assert c.layout_order == i



# -*- coding: utf-8 -*-

class StandardReprMixin(object):
    u"""
    Used to add an implementation of '__repr__' that is generally 
    useful for debugging.

    >>> class Foo(StandardReprMixin):
    ...     def __init__(self):
    ...         self.name = "blé"
    >>>
    >>> f = Foo()
    >>> f.frobble = 1
    >>> print f
    <Foo frobble=1 name='bl\\xc3\\xa9'>
    """
    def __repr__(self):
        return u"<%s %s>" % (self.__class__.__name__,
                             u' '.join(u"%s=%r" % (k,v) for (k,v) in sorted(self.__dict__.iteritems())
                                       if k != '__doc__'))

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



from django.db import models
from semanticeditor.fields import MultiSelectField
from django.conf import settings


template_list = [(f,n) for (f,n) in settings.CMS_TEMPLATES if f != settings.CMS_TEMPLATE_INHERITANCE_MAGIC]


class CssClassCategory(models.Model):
    name = models.CharField("Category name", max_length=255, unique=True)

    def __unicode__(self):
        return self.name


    class Meta:
        verbose_name = "CSS class category"
        verbose_name_plural = "CSS class categories"
        ordering = ('name',)


class CssClass(models.Model):
    category = models.ForeignKey(CssClassCategory, null=True, blank=True)
    name = models.CharField("CSS class name", max_length=255, unique=True,
                                  help_text="The name as it appears in the CSS file")
    verbose_name = models.CharField("Human name", max_length=255, blank=False, unique=True,
                                    help_text="User-visible name of the style.")
    description = models.TextField("Description", max_length=255, blank=True,
                                   help_text="This is displayed as a 'tooltip' "
                                   "when the user hovers over the name in the "
                                   "list of styles.  The data will be interpreted "
                                   "as raw HTML, so it can include an example.")
    templates = MultiSelectField("Templates", choices=template_list, blank=True,
                                 default="")

    allowed_elements = models.CharField("Allowed HTML elements", max_length=255,
                                        help_text="A space separated list of HTML "
                                        "element names.  Use 'newrow'/'newcol'/"
                                        "'innerrow'/'innercol' to indicate "
                                        "it can be applied to layout rows or columns ",
                                        default="h1 h2 h3 h4 h5 h6 p blockquote ul ol li newrow newcol")

    column_equiv = models.IntegerField("Column count equivalent", null=True, blank=True,
                                       help_text="For classes designed to be applied to "
                                       "columns only, this is the number of columns this "
                                       "should be considered as equivalent to. This can be "
                                       "useful for generating double width columns etc. "
                                       "within a column layout.")
    def __unicode__(self):
        return self.verbose_name

    class Meta:
        verbose_name = "CSS class"
        verbose_name_plural = "CSS classes"
        ordering = ('verbose_name',)


def get_classes(template):
    # Can't do filter in DB easily, because 'templates' is actually a comma
    # separated list in DB.
    classes = CssClass.objects.all().order_by('category__name', 'verbose_name')
    return filter(lambda c: template in c.templates, classes)



from cms.plugins.text import settings as text_settings
from cms.plugins.text.widgets.wymeditor_widget  import WYMEditor
from django import forms
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django.utils.translation.trans_real import get_language
import os

join = os.path.join

class SemanticEditor(WYMEditor):

    class Media:
        js = [join(settings.SEMANTICEDITOR_MEDIA_URL, path) for path in
              ('javascript/wymeditor/plugins/semantic/wymeditor.semantic.js',
               'javascript/json2.js',
               'javascript/jquery.query-2.1.7.js',
               )]

    def __init__(self, attrs=None, installed_plugins=None, page=None):
        self.page = page
        super(SemanticEditor, self).__init__(attrs=attrs, installed_plugins=installed_plugins)

    def render_additions(self, name, value, attrs=None):
        language = get_language()
        containers = getattr(settings, 'WYM_CONTAINERS', None)
        if containers is None:
            containers = text_settings.WYM_CONTAINERS
        context = {
            'name': name,
            'language': language,
            'SEMANTICEDITOR_MEDIA_URL': settings.SEMANTICEDITOR_MEDIA_URL,
            'CMS_MEDIA_URL': settings.CMS_MEDIA_URL,
            'WYM_TOOLS': mark_safe(text_settings.WYM_TOOLS),
            'WYM_CONTAINERS': mark_safe(containers),
            'WYM_CLASSES': mark_safe(text_settings.WYM_CLASSES),
            'WYM_STYLES': mark_safe(text_settings.WYM_STYLES),
            'WYM_STYLESHEET': mark_safe(text_settings.WYM_STYLESHEET),
            'installed_plugins': self.installed_plugins,
            'page': self.page,
        }

        return mark_safe(render_to_string(
            'semanticeditor/editorwidget.html', context))



"""
Functionality common to different parts of the HTML manipulation
(but not generic enough for utils)
"""

from lxml import etree as ET
from lxml.html import HTMLParser

from semanticeditor.definitions import IncorrectHeadings, BLOCKDEF, BLOCK_LEVEL_TRIM_LENGTH, HEADINGDEF
from semanticeditor.utils.datastructures import struct
from semanticeditor.utils.etree import flatten, get_depth, cleanup

### Structure related ###

class StructureItem(object):
    __metaclass__ = struct
    level = 0     #    level is the 'outline level' in the document i.e. an integer
    sect_id = ''  #    sect_id is a unique ID used for storing presentation information against
    name = ''     #    name is a user presentable name for the section
    tag = ''      #    tag is the HTML element e.g. h1
    node = None   #    node is the ElementTree node


def get_structure(root, assert_structure=False):
    """
    Return the structure nodes, as a list of StructureItems.  Structure nodes
    are nodes that can have commands or classes applied to them.
    """
    retval = []
    sect_ids = set()
    headings_used = False
    cur_level = 1
    last_heading_num = 0
    first_heading_level = 1

    # Pre-pass to get existing ids.
    for n in root.getiterator():
        if n.tag in BLOCKDEF:
            sect_id = n.get('id')
            if sect_id is not None:
                if sect_id in sect_ids:
                    # don't use duplicate ids.
                    del n.attrib['id']
                else:
                    # reserve
                    sect_ids.add(sect_id)

    for n in root.getiterator():
        if n.tag in BLOCKDEF:
            text = flatten(n)
            sect_id = n.get('id')
            if sect_id is None:
                sect_id = _make_sect_id(n.tag, sect_ids)
            sect_ids.add(sect_id)
            if n.tag in HEADINGDEF:
                name = text
                level = int(n.tag[1])
                cur_level = level
                if assert_structure:
                    if not headings_used:
                        first_heading_level = level
                    else:
                        if level < first_heading_level:
                            raise IncorrectHeadings("No heading can be higher than the first "
                                                    "heading, which was H%d." %
                                                    first_heading_level)

                    # Heading level should decrease or increase by no more than one.
                    if headings_used and level > last_heading_num + 1:
                        raise IncorrectHeadings('Heading "%(name)s" is level H%(foundnum)d, '
                                                'but it should be level H%(rightnum)d or less' %
                                                dict(name=name, foundnum=level,
                                                     rightnum=last_heading_num + 1))

                if name == "":
                    name = "?"

                last_heading_num = level
                headings_used = True
            else:
                name = text[0:BLOCK_LEVEL_TRIM_LENGTH]
                if name == '':
                    name = '?'
                else:
                    name = name + "..."

                # Paragraphs etc within a section should be indented
                # one further than the heading above them.
                if not headings_used:
                    level = 1
                else:
                    level = cur_level + 1

            # Level is adjusted so that e.g. H3 is level 1, if it is
            # the first to appear in the document.
            # It is also adjusted so that nested items (e.g. p in blockquote)
            # appear to be nested.
            nesting_level = get_depth(root, n) - 2
            retval.append(StructureItem(level=nesting_level + level - first_heading_level + 1,
                                        sect_id=sect_id,
                                        name=name,
                                        tag=n.tag.lower(),
                                        node=n))

    return retval


def _make_sect_id(tag, used_ids):
    i = 1
    while True:
        attempt = tag + "_" + str(i)
        if attempt not in used_ids:
            return attempt
        else:
            i += 1


def parse(content, clean=False):
    """
    Parses the HTML provided into an ElementTree.
    If 'clean' is True, lax parsing is done, the tree is cleaned
    of dirty user provided HTML
    """
    # We also use HTMLParser for 'strict', because the XML parser seems to eliminate
    # '\r' for some reason.
    tree = ET.fromstring(u'<html><body>' + content + u'</body></html>', parser=HTMLParser())
    if clean:
        from semanticeditor.clean import clean_tree
        clean_tree(tree)
    return tree


def html_extract(root):
    return ET.tostring(root).replace('<html>', '').replace('</html>', '').replace('<body>', '').replace('</body>', '').replace('<body/>','').replace("<head/>", "").replace("&#13;", "\r")


def get_classes_for_node(node):
    return filter(len, node.get('class', '').split(' '))


def get_classes_from_presinfo(presinfos):
    # Extract a list of classes from a list of PresentationInfo objects
    return [pi.name for pi in presinfos if pi.prestype == "class"]


def strip_presentation(tree):
    cleanup(tree, lambda t: t.tag == 'div')



from django.forms.models import ModelForm
from cms.plugins.text.models import Text
from django import forms
from semanticeditor.widgets import SemanticEditor

class SemanticTextForm(ModelForm):
    body = forms.CharField(widget=SemanticEditor())
    class Meta:
        model = Text
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')



"""
Utilities for cleaning user HTML
"""

from lxml import etree as ET
from pyquery import PyQuery as pq

from semanticeditor.common import html_extract, parse, get_classes_for_node
from semanticeditor.definitions import BLOCKDEF_SELECTOR, COMMANDS
from semanticeditor.utils.etree import get_parent, get_index, eliminate_tag, empty_text
from django.conf import settings

disallowed_elements = getattr(settings, "SEMANTICEDITOR_DISALLOWED_ELEMENTS", ['span', 'li p:only-child', 'table', 'tbody', 'thead', 'tr', 'td'])

def clean_tree(root):
    """
    Cleans dirty HTML from an ElementTree
    """
    initial_html = ET.tostring(root)
    body = root[0] # <html><body>
    # If there is text directly in body, it needs wrapping in a block element.
    _promote_child_text(body, 'p')

    # replace 'command' divs
    _remove_command_divs(body)

    # First replace divs
    _replace_block_elements(body)

    # Deal with nested 'p's and other elements.
    _clean_nested(body)

    doc = pq(root)
    doc('*').each(_clean_elem)
    doc('style').remove()
    doc('col').remove()

    def pull_up(n):
        p = get_parent(body, n)
        i = get_index(p, n)
        eliminate_tag(p, i)

    for x in disallowed_elements: 
        for n in doc(x):
            pull_up(n)
    # "li p:only-child" appears to be buggy.  It works like
    # "li p:only-descendent" or something.

    for x in ['strong', 'em', 'b', 'i']:
        for n in doc(x):
            if pq(n).is_(BLOCKDEF_SELECTOR):
                pull_up(n)

    # remove duplicate 'id' attributes.
    ids = [n.get('id', None) for n in doc('*[id]')]
    ids = [i for i in ids if i != "" and i != None]
    for i in set(ids):
        for j, node in enumerate(doc('#' + i)):
            if (j > 0): # skip the first one
                del node.attrib['id']

    doc('p + br').remove()
    for par in doc('p:empty'):
        if par.text is None  or par.text.strip() == "":
            par.getparent().remove(par)

    # Removed elements can give problems which need to be fixed again.  We keep
    # iterating through this until we get the same answer!
    output_html = ET.tostring(root)
    if initial_html == output_html:
        return
    else:
        clean_tree(root)


def clean_html(html):
    tree = parse(html, clean=True)
    return html_extract(tree)


def _clean_text(t):
    return t.replace(u'\xa0', u' ')


def _clean_elem(i, d):
    for x in ['style', 'class']:
        try:
            del d.attrib[x]
        except KeyError:
            pass
    for elem in d:
        if elem.text is not None:
            elem.text = _clean_text(elem.text)
        if elem.tail is not None:
            elem.tail = _clean_text(elem.tail)


def _promote_child_text(elem, tag):
    """
    Ensure any leading or trailing text directly as a child of elem is wrapped
    in a tag.
    """
    if not empty_text(elem.text):
        newtag = ET.Element(tag)
        newtag.text = elem.text
        elem.insert(0, newtag)
        elem.text = None

    if len(elem) > 0 and not empty_text(elem[-1].tail):
        newtag = ET.Element(tag)
        newtag.text = elem[-1].tail
        elem[-1].tail = None
        elem.append(newtag)


def _clean_nested(elem):
    for idx, child in reversed(list(enumerate(elem.getchildren()))):
        # (do it reversed so that indexes never change as we mutate children)
        _clean_nested(child)
        if child.tag == 'p' and elem.tag == 'p':
            eliminate_tag(elem, idx)


def _replace_block_elements(elem):
    for child in elem.getchildren():
        if child.tag == 'div':
            child.tag = 'p'
        _replace_block_elements(child)


def _remove_command_divs(elem):
    for child in reversed(elem.getchildren()):
        _remove_command_divs(child)
        if child.tag == 'div' or child.tag == 'p':
            classes = set(get_classes_for_node(child))
            if any(c.name in classes for c in COMMANDS):
                elem.remove(child)



from django import forms
from django.core import exceptions
from django.db import models
from django.utils.text import capfirst

# Thanks to danielroseman from djangosnippets.org for this code!

class MultiSelectFormField(forms.MultipleChoiceField):
    widget = forms.SelectMultiple

    def __init__(self, *args, **kwargs):
        self.max_choices = kwargs.pop('max_choices', 0)
        super(MultiSelectFormField, self).__init__(*args, **kwargs)

    def clean(self, value):
        if not value and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        if value and self.max_choices and len(value) > self.max_choices:
            raise forms.ValidationError('You must select a maximum of %s choice%s.'
                    % (apnumber(self.max_choices), pluralize(self.max_choices)))
        return value

    def widget_attrs(self, widget):
        return {'class':'checkboxlist'}

class MultiSelectField(models.Field):
    __metaclass__ = models.SubfieldBase

    def get_internal_type(self):
        return "TextField"

    def get_choices_default(self):
        return self.get_choices(include_blank=False)

    def _get_FIELD_display(self, field):
        value = getattr(self, field.attname)
        choicedict = dict(field.choices)

    def formfield(self, **kwargs):
        # don't call super, as that overrides default widget if it has choices
        defaults = {'required': not self.blank, 'label': capfirst(self.verbose_name), 
                    'help_text': self.help_text, 'choices':self.choices}
        if self.has_default():
            defaults['initial'] = self.get_default()
        defaults.update(kwargs)
        return MultiSelectFormField(**defaults)

    def get_db_prep_value(self, value, connection, prepared=False):
        if isinstance(value, basestring):
            return value
        elif isinstance(value, list):
            return ",".join(value)

    def to_python(self, value):
        if isinstance(value, list):
            return value
        elif value==None:
            return ''
        return value.split(",")

    def contribute_to_class(self, cls, name):
        super(MultiSelectField, self).contribute_to_class(cls, name)
        if self.choices:
            func = lambda self, fieldname = name, choicedict = dict(self.choices):",".join([choicedict.get(value,value) for value in getattr(self,fieldname)])
            setattr(cls, 'get_%s_display' % self.name, func)

    def validate(self, value, model_instance):
        choice_keys = set(k for k, v in self._choices)
        # value is a list of values
        for v in value:
            if v not in choice_keys:
                raise exceptions.ValidationError(self.error_messages['invalid_choice'] % value)


from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^semanticeditor\.fields\.MultiSelectField"])



from cms.models import Page
from cms.plugins.text.cms_plugins import TextPlugin
from cms.plugins.text.models import Text
from cms.plugins.text.forms import TextForm
from cms.plugins.text.utils import plugin_tags_to_user_html
from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from django.utils.translation import ugettext_lazy as _

from semanticeditor.widgets import SemanticEditor
from django.forms.fields import CharField

import re

class SemanticTextPlugin(TextPlugin):

    name = _("Text/layout")
    admin_preview = False

    # A lot of duplication from TextPlugin because get_form needs to find out
    # what page/template we are using, and pass that on to get_editor_widget

    def get_editor_widget(self, request, plugins, page):
        return SemanticEditor(installed_plugins=plugins,
                              page=page)

    def get_form_class(self, request, plugins, page):
        """
        Returns a subclass of Form to be used by this plugin
        """
        # We avoid mutating the Form declared above by subclassing
        class TextPluginForm(self.form):
            pass

        widget = self.get_editor_widget(request, plugins, page)
        TextPluginForm.declared_fields["body"] = CharField(widget=widget, required=False)
        return TextPluginForm

    def get_form(self, request, obj=None, **kwargs):

        page = None
        if obj:
            # if obj.placeholder.page, we must be editing a plugin belonging to a page; 
            placeholder = obj.placeholder
            field = placeholder._get_attached_field_name()
            model = placeholder._get_attached_model()
        
            # if not, we can find the object it belongs to via placeholder._get_attached_model()
        
            # all Arkestra models with placeholders will (should) have a get_website() method
            # for other models, that don't, we just do without
        
            try:
                page = obj.page or model.objects.get(**{field: obj.placeholder.id}).get_website()
            except AttributeError:
                import warnings
                warnings.warn("Couldn't work out get_website() for this item, which will result in problems with class list")


        plugins = plugin_pool.get_text_enabled_plugins(self.placeholder, page)
        form = self.get_form_class(request, plugins, page)
        kwargs['form'] = form # override standard form
        return super(TextPlugin, self).get_form(request, obj, **kwargs)

plugin_pool.register_plugin(SemanticTextPlugin)



# This module contains the 'public' API for parsing/formatting HTML,
# used by views.py

from semanticeditor.clean import clean_html
from semanticeditor.definitions import AllUserErrors, COMMANDS, PresentationInfo, PresentationClass
from semanticeditor.extract import extract_presentation, extract_structure
from semanticeditor.format import format_html, preview_html
from semanticeditor.models import get_classes



import re

from lxml import etree as ET

from semanticeditor.definitions import COMMANDS, SORTED_COMMANDS, NEWROW, NEWCOL, NEWINNERROW, NEWINNERCOL, TooManyColumns, BadStructure
from semanticeditor.common import get_classes_for_node, get_classes_from_presinfo
from semanticeditor.utils.etree import get_parent


### Layout details ###

# This is designed to be user supply-able if necessary

class LayoutDetailsBase(object):
    """
    Base class for strategy object used to define the details of
    CSS/HTML to be used when rendering a layout
    """
    # Inherit from this class if creating your own custom class.  LayoutDetails
    # provides a concrete implementation.

    def _raise_not_implemented(self):
        raise NotImplementedError()

    max_columns = property(_raise_not_implemented, doc="""Maximum number of columns to allow""")

    use_inner_column_div = property(_raise_not_implemented, doc="""True to wrap all column content in a inner div""")

    def row_classes(self, logical_column_count, actual_column_count):
        """
        Returns a list of CSS classes to be used for a row containing
        logical_column_count 'logical' columns, actual_column_count 'actual'
        columns.  'actual' columns are present in the HTML structure, but some
        might be, for example, double width, so are counted as two logical
        columns.
        """
        raise NotImplementedError()

    def column_classes(self, logical_column_num, actual_column_num, logical_column_count, actual_column_count):
        """
        Returns a list of CSS classes to be used for a column which is number
        column_num out of column_count.  (see above regarding logical/actual)
        """
        raise NotImplementedError()

    def is_row_class(self, class_):
        """
        Returns true if the class (a string) corresponds to a CSS class used for
        a row.
        """
        raise NotImplementedError

    def is_column_class(self, class_):
        """
        Returns true if the class (a string) corresponds to a CSS class used for
        a column.
        """
        raise NotImplementedError()

    def outer_column_classes(self, presinfo):
        """
        Given a list a PresentationInfo objects, return the ones that should be
        applied to the outer column div.
        """
        if not self.use_inner_column_div:
            return presinfo
        else:
            raise NotImplementedError()

    def inner_column_classes(self, presinfo):
        """
        Given a list a PresentationInfo objects, return the ones that should be
        applied to the inner column div.  (Never called if use_inner_column_div
        = False)
        """
        raise NotImplementedError()

    # Hacks, optional
    def format_pre_parse_hacks(self, html, styleinfo):
        """
        For formatting, applies hacks to unformatted HTML before parsing,
        returns HTML to be used.
        """
        return html

    def format_post_parse_hacks(self, tree, styleinfo):
        """
        For formatting, applies hacks to tree after parsing, returns new tree to
        be used.
        """
        return tree

    def format_structure_hacks(self, structure, styleinfo):
        """
        For formatting, given a list of StructureItems and a list of
        PresentationInfos, applies hacks and returns new structure to be used.
        """
        return structure

    def format_post_layout_hacks(self, tree, structure, styleinfo):
        """
        For formatting, given the tree after layout, the structure and style
        info, apply hacks and return a new tree.
        """
        return tree

    def extract_pre_parse_hacks(self, html):
        """
        For extracting presentation info, applies hacks to formatted HTML before
        parsing, and returns HTML to be used.
        """
        return html

    def extract_post_parse_hacks(self, tree):
        """
        For extracting presentation info, applies hacks to parse tree before
        after parsing, and returns tree.
        """
        return tree

    def extract_structure_hacks(self, structure):
        """
        For extracting presentation info, given a list of StructureItems,
        applies hacks and returns new structure to be used.
        """
        return structure

class LayoutDetails(LayoutDetailsBase):
    """
    Strategy object used for defining the details of CSS/HTML to be used when
    rendering a Layout.  This is a concrete implementation.
    """
    ROW_CLASS = "row"
    COLUMN_CLASS = "column"

    max_columns = 6

    use_inner_column_div = True

    def row_classes(self, logical_column_count, actual_column_count):
        retval = [self.ROW_CLASS]
        if actual_column_count > 1:
            retval.append("columns%d" % logical_column_count)
        return retval

    def column_classes(self, logical_column_num, actual_column_num, logical_column_count, actual_column_count):
        if actual_column_count == 1:
            # No classes
            return []
        retval = [self.COLUMN_CLASS]
        if actual_column_num == 1:
            retval.append("firstcolumn")
        if actual_column_num == actual_column_count:
            retval.append("lastcolumn")
        return retval

    def is_row_class(self, class_):
        return class_ == self.ROW_CLASS or re.match(r'^columns\d+$', class_)

    def is_column_class(self, class_):
        return class_ == self.COLUMN_CLASS or re.match(r'^(first|last)column$', class_)

    def outer_column_classes(self, presinfo):
        return [pi for pi in presinfo if pi.column_equiv is not None]

    def inner_column_classes(self, presinfo):
        return [pi for pi in presinfo if pi.column_equiv is None]

    # Hacks
    def format_post_layout_hacks(self, tree, structure, styleinfo):
        # WYMEditor cannot insert divs. This is a workaround
        for n in tree.getiterator():
            if n.tag == 'p' and ('div' in get_classes_for_node(n)):
                n.tag = 'div'
            if n.tag == 'p':
                # If only child element is a plugin object, convert to
                # a div.
                # NB: current implementation of plugin objects is that they
                # are represented by an image in the editor.  Our code has to
                # run before these are converted, so we have to work with this
                # implementation detail.
                children = n.getchildren()
                if ((n.text is None or n.text.strip() == "")
                    and len(children) == 1
                    and children[0].tag == "img"
                    and (children[0].tail is None or children[0].tail.strip() == "")
                    and children[0].attrib.get('id', '').startswith("plugin_obj")):
                        n.tag = 'div'
                        # Add 'div' to list of classes
                        # This handles the reverse transform for us:
                        n.attrib['class'] = ' '.join(n.attrib.get('class', '').split(' ') + ['div']).strip()
        return tree

    def extract_post_parse_hacks(self, tree):
        # inverse part of above workaround
        for n in tree.getiterator():
            if n.tag == 'div' and ('div' in get_classes_for_node(n)):
                n.tag = 'p'
        return tree


def get_layout_details_strategy():
    # TODO - make configurable
    return LayoutDetails()



### Creating layouts ###

# Simple wrapper for nodes to provide as_nodes() method.
class NodeContent(object):
    def __init__(self, node):
        self.node = node

    def as_nodes(self, layout_strategy):
        return [self.node]

# Simple container for whole layout.
class Layout(object):
    # True if the structure corresponding to this command allows content to be
    # embedded directly.
    accepts_content = True

    def __init__(self):
        self.content = []

# LayoutRow contains a list of columns, and a list of PresentationInfo objects
class LayoutRow(object):

    accepts_content = False

    def __init__(self, presinfo=None):
        if presinfo is None:
            presinfo = []
        self.content = []
        self.presinfo = presinfo

    def column_count(self):
        """
        Get the number of logical columns.
        """
        return sum(_layout_column_width(c) for c in self.content)

    def as_nodes(self, layout_strategy):
        """
        Returns layout as a list of ElementTree nodes
        """
        # Row
        logical_column_count = self.column_count()
        actual_column_count = len(self.content)
        rowdiv = ET.Element('div')
        classes = layout_strategy.row_classes(logical_column_count, actual_column_count) + get_classes_from_presinfo(self.presinfo)
        if classes:
            rowdiv.set('class', ' '.join(classes))

        # Columns
        logical_column_num = 1
        for i, col in  enumerate(self.content):
            coldiv = ET.Element('div')
            classes = layout_strategy.column_classes(logical_column_num,
                                                     i + 1,
                                                     logical_column_count,
                                                     actual_column_count) + \
                    get_classes_from_presinfo(layout_strategy.outer_column_classes(col.presinfo))
            if classes:
                coldiv.set('class', ' '.join(classes))
            if layout_strategy.use_inner_column_div:
                contentdiv = ET.Element('div')
                coldiv.append(contentdiv)
                inner_classes = get_classes_from_presinfo(layout_strategy.inner_column_classes(col.presinfo))
                if inner_classes:
                    contentdiv.set('class', ' '.join(inner_classes))
            else:
                contentdiv = coldiv
            for n in col.content:
                contentdiv.extend(n.as_nodes(layout_strategy))
            rowdiv.append(coldiv)

            logical_column_num += _layout_column_width(col)
        return [rowdiv]

# LayoutColumn contains a list of content, and a list of PresentationInfo objects.
class LayoutColumn(object):

    accepts_content = True

    def __init__(self, presinfo=None):
        if presinfo is None:
            presinfo = []
        self.content = []
        self.presinfo = presinfo


### Structures for commands ###

LAYOUT_STRUCTURES = {
    NEWROW: LayoutRow,
    NEWCOL: LayoutColumn,
    NEWINNERROW: LayoutRow,
    NEWINNERCOL: LayoutColumn,
}


def _layout_column_width(col):
    """
    Returns the logical column width of a column
    """
    column_equivs = [pi.column_equiv for pi in col.presinfo if pi.column_equiv is not None]
    if len(column_equivs) > 0:
        # assume user has not done something silly like put
        # *2* column_equiv classes on a column
        return column_equivs[0]
    else:
        return 1

def _find_layout_commands(root, structure, styleinfo):
    # Layout commands are not stored against normal sections,
    # but have their own entry in the section list, using an id
    # of 'newrow_' or 'newcol_' + id of block they precede.

    sect_dict = dict((s.sect_id, s) for s in structure)
    command_info = {}
    for c in COMMANDS:
        # for each command, store a dictionary that is
        # key = sect_id, val = [PresentationInfo]
        command_info[c.name] = {}

    for sect_id, presinfo in styleinfo.items():
        for c in COMMANDS:
            if sect_id.startswith(c.prefix):
                real_sect_id = sect_id[len(c.prefix):]
                sect = sect_dict.get(real_sect_id)
                if sect is not None:
                    parent = get_parent(root, sect.node)
                    if not _is_root(parent):
                        raise BadStructure("Section \"%(name)s\" is not at the top level of the"
                                           " document, and therefore cannot have a column"
                                           " structure applied to it.  Please move the"
                                           " '%(commandname)s' command to a top level element." %
                                           dict(name=sect.name,
                                                commandname=c.name))

                command_info[c.name][real_sect_id] = presinfo

    return command_info

def create_layout(root, styleinfo, structure):
    # Find the layout commands
    command_info = _find_layout_commands(root, structure, styleinfo)

    # Build a Layout structure.
    layout = Layout()

    # Get nodes
    nodes = root.getchildren()
    if nodes and nodes[0].tag == 'body':
        nodes = nodes[0].getchildren()

    sect_dict = dict((si.node, si) for si in structure)

    # Now build a layout. At each point, we need to:
    # - append nodes to current container
    # - keep track of all containers, so we can respond to new row/column commands.

    containers = [layout]
    current_level = -1

    for node in nodes:
        si = sect_dict.get(node)
        if si:
            for command in SORTED_COMMANDS:
                presinfo = command_info[command.name].get(si.sect_id)
                if presinfo is not None:
                    # We have the command.

                    # First, need to work out what level it is on:
                    command_level = command.layout_order

                    if command_level > current_level + 1:
                        lowercommand = SORTED_COMMANDS[command_level-1]
                        raise BadStructure('Section "%(sect)s" has command "%(command)s" '
                                           'but there needs to be a "%(lowercommand)s" '
                                           'command first.' %
                                           dict(sect=si.name,
                                                command=command.verbose_name,
                                                lowercommand=lowercommand.verbose_name)
                                           )

                    if command_level <= current_level:
                        # Need to pop of list of containers so that the next
                        # container goes on the right parent.

                        # Pop as many containers as necessary
                        for i in xrange(current_level - command_level + 1):
                            containers.pop()

                    # Make new container
                    layout_container = LAYOUT_STRUCTURES[command](presinfo=presinfo)
                    containers[-1].content.append(layout_container)
                    containers.append(layout_container)

                    current_level = command_level

        # Deal with the nodes

        # Check whether we can add them here.
        if not containers[-1].accepts_content:
            # can't append content to this. We infer the presence of the next
            # structure down.
            current_level += 1
            next_command = SORTED_COMMANDS[current_level]
            layout_container = LAYOUT_STRUCTURES[next_command]()
            # Currently this will always produce a command that accepts content
            assert layout_container.accepts_content
            containers[-1].content.append(layout_container)
            containers.append(layout_container)

        # Add the content nodes.
        containers[-1].content.append(NodeContent(node))

    # Need to remove empty layout structures.
    _trim_empty_layout(layout)

    return layout

def _trim_empty_layout(layout):
    for i, l in reversed(list(enumerate(layout.content))):
        if hasattr(l, 'content'):
            if not l.content:
                # l has nothing, so remove from parent.
                layout.content[i:i+1] = []
            else:
                _trim_empty_layout(l)

def check_layout(row, structure, layout_strategy, sect_dict=None):
    if sect_dict is None:
        sect_dict = dict((si.node, si) for si in structure)
    max_cols = layout_strategy.max_columns

    # Cope with NodeContent:
    if not hasattr(row, 'content'):
        return

    if row.column_count() > max_cols:
        # Because columns can be multiple width, we can't easily work out
        # which column needs to be moved, so just refer user to whole
        # section.

        nodes = row.content[0].content[0].as_nodes(layout_strategy)
        while True:
            # nodes[0] might be a div created for layout.  If so, it won't
            # be in sect_dict. But one of its children will be.
            sect = sect_dict.get(nodes[0], None)
            if sect is not None:
                break
            else:
                nodes = nodes[0]

        raise TooManyColumns("The maximum number of columns is %(max)d. "
                             "Please adjust columns in section '%(name)s'." %
                             dict(max=max_cols, name=sect.name))

    for col in row.content:
        # Check nested layouts.
        for content in col.content:
            check_layout(content, structure, layout_strategy, sect_dict=sect_dict)


def _is_root(node):
    return node.tag == 'html' or node.tag == 'body'







from south.db import db
from django.db import models
from semanticeditor.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'CssClass.templates'
        db.add_column('semanticeditor_cssclass', 'templates', orm['semanticeditor.cssclass:templates'])
        
    
    
    def backwards(self, orm):
        
        # Deleting field 'CssClass.templates'
        db.delete_column('semanticeditor_cssclass', 'templates')
        
    
    
    models = {
        'semanticeditor.cssclass': {
            'allowed_elements': ('django.db.models.fields.CharField', [], {'default': "'h1 h2 h3 h4 h5 h6 p blockquote ul li row column'", 'max_length': '255'}),
            'column_equiv': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'unique': 'True'}),
            'templates': ('semanticeditor.fields.MultiSelectField', ['"Templates"'], {'default': "''", 'blank': 'True'}),
            'verbose_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'unique': 'True'})
        }
    }
    
    complete_apps = ['semanticeditor']




from south.db import db
from django.db import models
from semanticeditor.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'CssClass'
        db.create_table('semanticeditor_cssclass', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField("CSS class name", unique=True, max_length=255)),
            ('verbose_name', models.CharField("Human name", unique=True, max_length=255, blank=False)),
            ('description', models.TextField("Description", max_length=255, blank=True)),
            ('allowed_elements', models.CharField("Allowed HTML elements", default='h1 h2 h3 h4 h5 h6 p blockquote ul li row column', max_length=255)),
            ('column_equiv', models.IntegerField("Column count equivalent", null=True, blank=True)),
        ))
        db.send_create_signal('semanticeditor', ['CssClass'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'CssClass'
        db.delete_table('semanticeditor_cssclass')
        
    
    models = {
        'semanticeditor.cssclass': {
            'allowed_elements': ('django.db.models.fields.CharField', [], {'default': "'h1 h2 h3 h4 h5 h6 p blockquote ul li row column'", 'max_length': '255'}),
            'column_equiv': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'unique': 'True'}),
            'verbose_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'unique': 'True'})
        }
    }

    complete_apps = ['semanticeditor']



from django.conf.urls.defaults import patterns, url
from semanticeditor.views import *

urlpatterns = patterns('',
    url(r'retrieve_styles/', retrieve_styles, name="semantic.retrieve_styles"),
    url(r'retrieve_commands/', retrieve_commands, name="semantic.retrieve_commands"),
    url(r'separate_presentation/', separate_presentation, name="semantic.separate_presentation"),
    url(r'combine_presentation/', combine_presentation, name="semantic.combine_presentation"),
    url(r'clean_html/', clean_html_view, name="semantic.clean_html"),
    url(r'preview/', preview, name="semantic.preview"),
)



"""
Use the 'struct' function (see below), or use the __metaclass__
attribute with class definition syntax.

>>> class Animal(object):
...    __metaclass__ = struct
...    name = ""
...    weight = 0
...    can_fly = False
>>> Animal(name="Pig")
<Animal can_fly=False name='Pig' weight=0>

Subclass to add more attributes or override
existing ones.  Multiple bases can be used.

>>> class Bird(Animal):
...    __metaclass__ = struct
...    can_fly = True
...    sings = None
>>> Bird(sings=True)
<Bird can_fly=True name='' sings=True weight=0>

You can subclass to add methods like this:

>>> class SelfDescribingBird(Bird):
...     def describe(self):
...         if self.sings:
...             print "I can sing"
>>> SelfDescribingBird(sings=True).describe()
I can sing

"""

from semanticeditor.utils.mixins import StandardReprMixin


def struct(name, bases, attrdict):
    """
    Returns a class that can be used as a simple struct, with default
    attributes defined in 'attrdict'.

    >>> Person = struct("Person", (), dict(age=0, name=""))
    >>> Person
    <class '__main__.Person'>
    >>> Person()
    <Person age=0 name=''>
    >>> Person(name="Joe")
    <Person age=0 name='Joe'>
    >>> Person(name="Fred", age=1)
    <Person age=1 name='Fred'>
    >>> Person(name="Fred", value=0, stupidity=1000)
    Traceback (most recent call last):
        ...
    TypeError: Person does not have attribute(s): stupidity, value

    >>> GenderedObject = struct("GenderedObject", (), dict(sex=None))

    If the default attribute is mutable, use a callable (e.g. a
    lambda, a class or a factory function) to provide it, otherwise it
    will be shared between instances.  Use the bases argument to
    inherit attribute definitions from other structs.

    >>> Parent = struct("Parent", (Person, GenderedObject), dict(children=list))
    >>> Parent(name="Fred", sex="male")
    <Parent age=0 children=[] name='Fred' sex='male'>
    """
    # In case '__metaclass__ = struct' is set directly, need these:
    attrdict.pop('__metaclass__', None)
    attrdict.pop('__module__', None)

    # Find allowed attributes of all bases
    attrs = {}
    for d in filter(lambda x: x is not None,
                    (getattr(b, '_allowed_attrs', None) for b in bases)):
        attrs.update(d)
    attrs.update(attrdict)

    def __init__(self, **kwargs):
        for k, v in attrs.items():
            try:
                val = kwargs.pop(k)
            except KeyError, e:
                val = v
                if callable(val):
                    val = val()
            setattr(self, k, val)
        if kwargs:
            raise TypeError("%s does not have attribute(s): %s" % \
                                (name, ", ".join(sorted(kwargs.keys()))))

    # we have no need for 'real' bases, and they only confuse things:
    bases = (StandardReprMixin,)
    return type(name, bases, {'__init__': __init__,
                              '_allowed_attrs': attrs})

# We could also define a 'Struct' class which has '__metaclass__ = StructBase
# Defining StructBase, however, is more tricky, and can't resuse
# the existing 'struct' function. (You end up needing to do:
#   s = struct(...)
#   s.__class__ = struct
# which does not work.)
# It also makes it harder to define normal subclasses that are just
# using the __init__ functionality provided by the 'struct' type.


def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'CssClassCategory'
        db.create_table('semanticeditor_cssclasscategory', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('semanticeditor', ['CssClassCategory'])


    def backwards(self, orm):
        
        # Deleting model 'CssClassCategory'
        db.delete_table('semanticeditor_cssclasscategory')


    models = {
        'semanticeditor.cssclass': {
            'Meta': {'ordering': "('verbose_name',)", 'object_name': 'CssClass'},
            'allowed_elements': ('django.db.models.fields.CharField', [], {'default': "'h1 h2 h3 h4 h5 h6 p blockquote ul ol li newrow newcol'", 'max_length': '255'}),
            'column_equiv': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'templates': ('semanticeditor.fields.MultiSelectField', [], {'default': "''", 'blank': 'True'}),
            'verbose_name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'semanticeditor.cssclasscategory': {
            'Meta': {'ordering': "('name',)", 'object_name': 'CssClassCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['semanticeditor']



"""
Extract simple HTML and presentation info from combined HTML
"""

from semanticeditor.common import parse, get_structure, get_classes_for_node, html_extract, strip_presentation
from semanticeditor.definitions import PresentationClass, NEWROW, NEWCOL, NEWINNERROW, NEWINNERCOL
from semanticeditor.layout import get_layout_details_strategy
from semanticeditor.utils.etree import get_parent, get_index
from semanticeditor.utils.general import any


def extract_structure(content):
    """
    Extracts H1, H2, etc headings, and other block level elements and
    returns a list of tuples containing (level, name, tag)
    """
    # This function is no longer used externally, but it has tests
    # against it that are useful at checking the behaviour of get_structure
    tree = parse(content, clean=True)
    structure = get_structure(tree, assert_structure=True)
    return structure


def extract_presentation(html):
    """
    Takes HTML with formatting applied and returns presentation elements (a
    dictionary with keys = section names, values = set of classes/commands) and
    the HTML without formatting (ready to be used in an editor)
    """
    # TODO: this function is not brilliantly well defined e.g.  should
    # there be an entry in the dictionary for sections with no
    # formatting?  This does not affect functionality, but it does
    # affect tests.
    layout_strategy = get_layout_details_strategy()
    html = layout_strategy.extract_pre_parse_hacks(html)
    root = parse(html, clean=False) # it's important we don't clean.
    root = layout_strategy.extract_post_parse_hacks(root)
    structure = get_structure(root)
    structure = layout_strategy.extract_structure_hacks(structure)
    pres = {}
    layout_commands = find_all_layout_nodes(root, layout_strategy)
    for si in structure:
        pres[si.sect_id] = set()

        # Section - extract classes
        for c in get_classes_for_node(si.node):
            pres[si.sect_id].add(PresentationClass(c))
            if 'class' in si.node.attrib:
                del si.node.attrib['class']

        # Add custom ids.  These are only for purpose of editing,
        # and will be removed again at end of format_html
        si.node.set('id', si.sect_id)

        # Now, deal with layout divs for this structure item
        cmd_pairs = layout_commands.get(si.node, [])
        for cmd, div_node in cmd_pairs:
            # Need to create another entry in pres
            pres_name = cmd.prefix + si.sect_id
            cmd_classes = set()

            # Find the classes that correspond to PresentationClass objects and
            # add them.
            node_classes = set(get_classes_for_node(div_node))
            if cmd in (NEWROW, NEWINNERROW):
                filterfunc = layout_strategy.is_row_class
            else:
                filterfunc = layout_strategy.is_column_class
                # Need the classes from the inner column div
                children = div_node.getchildren()
                if len(children) > 0 and children[0].tag == 'div':
                    node_classes |= set(get_classes_for_node(children[0]))

            for c in node_classes:
                if not filterfunc(c):
                    cmd_classes.add(PresentationClass(c))

            cmd_classes.add(cmd) # not strictly necessary, but helps testing
            pres[pres_name] = cmd_classes

    strip_presentation(root)
    out_html = html_extract(root)

    return (pres, out_html)


def find_all_layout_nodes(node, layout_strategy, depth=0, current_stack=None,
                          layout_divs=None, layout_commands=None):
    """
    Finds all the layout divs, and the content node they correspond to.
    Returns a dictionary:
       content node: [stack of (command, command node)],
    """
    if layout_commands is None:
        # Rather than messing with lots of return values, we define
        # a containers for collecting the results.
        layout_commands = {}

    if current_stack is None:
        current_stack = []

    children = node.getchildren()

    # Find out the command that corresponds to curent node
    if node.tag == 'div':
        c_classes = get_classes_for_node(node)
        is_col = any(layout_strategy.is_column_class(c) for c in c_classes)
        is_row = any(layout_strategy.is_row_class(c) for c in c_classes)
        if not is_col and len(current_stack) > 0 and \
                current_stack[-1][0] in (NEWROW, NEWINNERROW):
            # A div inside a row is always a column
            is_col = True
        if is_col:
            if depth == 0:
                current_stack.append((NEWCOL, node))
                depth += 1
            else:
                current_stack.append((NEWINNERCOL, node))
        elif is_row:
            current_stack.append((NEWROW if depth == 0 else NEWINNERROW, node))

    else:
        is_row, is_col = False, False


    # Is the current node the one the stack applies to?
    if not (is_col or is_row) and len(current_stack) > 0:
        # If this is the additional inner column div, we want to skip it
        if node.tag != 'div':
            layout_commands[node] = current_stack
            current_stack = []

    # The current stack of commands applies to the first content node that is a
    # descendent of them. We only look in first child, recursively.
    for i, child in enumerate(children):
        if i == 0:
            # Look for content node
            find_all_layout_nodes(child, layout_strategy, depth=depth,
                                  current_stack=current_stack,
                                  layout_commands=layout_commands)
        else:
            # Not the first child, so we reset the current_stack
            find_all_layout_nodes(child, layout_strategy, depth=depth,
                                  current_stack=[],
                                  layout_commands=layout_commands)

    return layout_commands



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def find_package_data(pkg, filetypes):
    import glob
    import itertools

    out = []
    for f in filetypes:
        for x in range(0, 20):
            pattern = pkg + '/' + ('*/' * x) + f
            out.extend([p[len(pkg)+1:] for p in glob.glob(pattern)])
    return out


setup(
    name = "semanticeditor",
    version = "0.2.1",
    description = "A Django CMS plugin for editing text with presentation and layout in a semantic way.",
    long_description = (
        read('README.rst') + '\n\n' + read('CHANGES.rst')
        ),
    url = 'https://bitbucket.org/spookylukey/semanticeditor/',
    license = 'BSD',
    author = 'Luke Plant',
    author_email = 'L.Plant.98@cantab.net',
    packages = find_packages(),
    package_data = {
        'semanticeditor': find_package_data('semanticeditor', ['*.js', '*.html', '*.css', '*.png'])
        },
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires = ['lxml >= 2.2.4', 'pyquery >= 0.6.1', 'django-cms >= 2.0'],
)



from django.contrib import admin
from semanticeditor.models import CssClass, CssClassCategory

class CssClassAdminInline(admin.StackedInline):
    model = CssClass
    extra = 0
    fieldsets = (
        (None, {
            'fields': (
                ('name', 'verbose_name'),
                ('description', 'templates',),
                ('allowed_elements', 'column_equiv'),
            ),
        }),
    )

class CssClassAdmin(admin.ModelAdmin):
    list_display = ('verbose_name', 'name', 'category', 'allowed_elements')
    list_editable = ('name', 'category', 'allowed_elements')

class CssClassCategoryAdmin(admin.ModelAdmin):
    inlines = [CssClassAdminInline,]
    
admin.site.register(CssClass, CssClassAdmin)
admin.site.register(CssClassCategory, CssClassCategoryAdmin)


from cms.models import Page
from django.http import HttpResponse
from django.utils import simplejson
from django.core.mail import mail_admins
from django.conf import settings
from django.utils.translation import ugettext as _
from semanticeditor.api import extract_presentation, format_html, preview_html, AllUserErrors, COMMANDS, PresentationInfo, PresentationClass, clean_html, get_classes
from semanticeditor.models import CssClass
import sys
try:
    from functools import wraps
except ImportError:
    from django.utils.functional import wraps  # Python 2.3, 2.4 fallback.


def json_view(func):
    """
    Use this decorator on a function that takes a request and returns
    a dictionary of values in order to create a view that handles
    errors and return JSON.

    The dictionary should be in this standard format:

    {'result': 'ok',
     'value': your_actual_value
    }

    or

    {'result': 'usererror',
     'message': an_error_message

    or

    {'result': 'error',
     'message': an_error_message
    }
    """
    def wrapper(request, *a, **kw):
        response = None
        try:
            response = func(request, *a, **kw)
        except Exception, e:
            # Mail the admins with the error
            exc_info = sys.exc_info()
            subject = 'JSON view error: %s' % request.path
            try:
                request_repr = repr(request)
            except:
                request_repr = 'Request repr() unavailable'
            import traceback
            message = 'Traceback:\n%s\n\nRequest:\n%s' % (
                '\n'.join(traceback.format_exception(*exc_info)),
                request_repr,
                )
            mail_admins(subject, message, fail_silently=True)

            # Come what may, we're returning JSON.
            if hasattr(e, 'message'):
                msg = e.message
            else:
                msg = _('Internal error')+': '+ str(e)
            response = error(msg)

        json = simplejson.dumps(response)
        return HttpResponse(json, mimetype='application/json')

    return wraps(func)(wrapper)


def error(msg):
    """
    Standard error result - for internal errors
    """
    return dict(result='error', message=msg)


def failure(msg):
    """
    Standard failure result
    """
    return dict(result='usererror', message=msg)


def success(value):
    """
    Standard success result
    """
    return dict(result='ok', value=value)


# Anything that depends on values the user may have entered and might
# contain 'errors' should use this, normally passing in AllUserErrors
# as 'exceptions'.  'server' errors are handled by @json_view
def graceful_errors(exceptions, callback):
    """
    Retrieve a value from a callback, handling the exceptions that
    are passed in, and returning in standard formats
    """
    try:
        val = callback()
    except exceptions, e:
        return failure(e.args[0])
    return success(val)


def PI_to_dict(pi):
    """
    Converts a PresentationInfo to a dictionary
    for use client side
    """
    d = {}
    # We need to filter out things that are server side only and can't be
    # turned into JSON.
    allowed = set([str, unicode, int, bool, dict, list])
    for k, v in pi.__dict__.items():
        if type(v) in allowed:
            d[k] = v
    return d


def dict_to_PI(d, classes):
    """
    Convert a dictionary to a PresentationInfo,
    using a pre-fetched dictionary of CssClass objects
    """
    if d['prestype'] == 'command':
        return PresentationInfo(prestype=d['prestype'], name=d['name'])
    else:
        c = classes.get(d['name'])
        if c is None:
            return None
        else:
            return css_class_to_presentation_class(c)


def css_class_to_presentation_class(c):
    return PresentationClass(c.name,
                             verbose_name=c.verbose_name,
                             description=c.description,
                             allowed_elements=c.allowed_elements.lower().split(' '),
                             column_equiv=c.column_equiv,
                             category=c.category.name if c.category is not None else None)


@json_view
def retrieve_styles(request):
    template = request.GET['template']
    page_id = request.GET['page_id']
    if template == settings.CMS_TEMPLATE_INHERITANCE_MAGIC:
        # Need to look up page to find out what template to use
        p = Page.objects.get(pk=page_id)
        template = p.get_template()
    classes = get_classes(template)
    retval = map(css_class_to_presentation_class,
                 classes)
    return success(map(PI_to_dict,retval))


@json_view
def retrieve_commands(request):
    return success(map(PI_to_dict, COMMANDS))


@json_view
def separate_presentation(request):
    """
    Returns a JSON object:
     { presentation: <dictionary of presentation info from html>
       html: <input html stripped of presentation>
     }
    """
    data = request.POST.get('html','')

    def _handled():
        pres, html = extract_presentation(data)
        # Rewrite pres so that we can serialise it to JSON
        pres2 = {}
        for k, v in pres.items():
            pres2[k] = [PI_to_dict(p) for p in v]
        return dict(presentation=pres2,
                    html=html)

    return graceful_errors(AllUserErrors, _handled)


def _convert_pres(pres):
    # Convert dictionaries into PresentationInfo classes. We need actual
    # CssClass instances in order to be able to restore column_equiv and
    # allowed_elements info
    classes = dict((c.name, c) for c in CssClass.objects.all())
    retval = {}
    for k, v in pres.items():
        # v is list of PI dicts
        newlist = []
        for i, item in enumerate(v):
            pi = dict_to_PI(item, classes)
            # if CssClass was removed from DB, pi can be None
            if pi is not None:
                newlist.append(pi)
        retval[k]= newlist
    return retval


@json_view
def combine_presentation(request):
    """
    Combines submitted 'html' and 'presentation' data,
    returning a dictionary containing { html: <combined html> }
    """
    html = request.POST.get('html', '')
    presentation = request.POST.get('presentation', '{}')
    presentation = simplejson.loads(presentation)
    presentation = _convert_pres(presentation)
    return graceful_errors(AllUserErrors, lambda: dict(html=format_html(html, presentation, pretty_print=True)))


@json_view
def preview(request):
    html = request.POST.get('html', '')
    presentation = request.POST.get('presentation', '{}')
    presentation = simplejson.loads(presentation)
    presentation = _convert_pres(presentation)

    return graceful_errors(AllUserErrors, lambda: dict(html=preview_html(html, presentation)))


@json_view
def clean_html_view(request):
    html = request.POST.get('html', '')
    return graceful_errors(AllUserErrors, lambda: dict(html=clean_html(html)))



"""
etree/ElementTree utils
"""

# ElementTree utilities.  Lots of these are pinched
# from proposed 'ElementLib' module on effbot.

def textjoin(a, b):
    a = a or ''
    b = b or ''
    return a + b

def cleanup(elem, filter):
    """
    Removes start and stop tags for any element for which the filter
    function returns True.  If you want to remove the entire element,
    including all subelements, use the 'clear' method inside the
    filter callable.

    Note that this function modifies the tree in place.
    @param elem An element tree.
    @param filter An filter function.  This should be a callable that
    takes an element as its single argument, and returns True if the
    element should be cleaned.
    """

    out = []
    for e in elem:
        cleanup(e, filter)
        if filter(e):
            if e.text:
                if out:
                    out[-1].tail = textjoin(out[-1].tail, e.text)
                else:
                    elem.text = textjoin(elem.text, e.text)
            out.extend(e)
            if e.tail:
                if out:
                    out[-1].tail = textjoin(out[-1].tail, e.tail)
                else:
                    elem.text = textjoin(elem.text, e.tail)
        else:
            out.append(e)
    elem[:] = out

def flatten(elem):
    text = elem.text or ""
    for e in elem:
        text += flatten(e)
        if e.tail:
            text += e.tail
    return text

def get_parent(topnode, elem):
    """
    Return the parent of 'elem'.

    topnode is the node to start searching from
    """
    for n in topnode.getiterator():
        if elem in n.getchildren():
            return n
    return None

def get_depth(topnode, elem, _start=0):
    """
    Returns the depth of elem in the tree, 0 for root node
    """
    if elem is topnode:
        return _start
    for n in topnode.getchildren():
        d = get_depth(n, elem, _start + 1)
        if d is not None:
            return d
    return None

def get_index(parent, elem):
    """
    Return the index of elem in parent's children
    """
    return list(parent.getchildren()).index(elem)

def indent(elem, level=0):
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for child in elem:
            indent(child, level+1)
        if not child.tail or not child.tail.strip():
            child.tail = i
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i

def eliminate_tag(parent, index):
    """
    Eliminates the tag from node at index 'index' from the parent.  The contents
    are pulled up into parent.
    """
    elem = parent[index]

    first = index == 0
    last = index == len(parent) - 1

    # 'text'
    if first:
        # 'text' merges with parents.
        parent.text = textjoin(parent.text, elem.text)
    else:
        # 'text' merges with tail of previous sibling
        prev = parent[index-1]
        prev.tail = textjoin(prev.tail, elem.text)
    # 'tail'
    if len(elem.getchildren()) > 0:
        # tail always goes on last child's tail
        elem[-1].tail = textjoin(elem[-1].tail, elem.tail)
    else:
        if first:
            # tail goes on parents text
            parent.text = textjoin(parent.text, elem.tail)
        elif last: # (and not first)
            prev = parent[index-1]
            prev.tail = textjoin(prev.tail, elem.tail)
        else:
            next = parent[index+1]
            next.text = textjoin(elem.tail, next.text)

    # Replace element with its children
    parent.remove(elem)
    for c in reversed(elem.getchildren()):
        parent.insert(index, c)


def empty_text(txt):
    """
    Returns True is the some text is considered empty
    """
    return txt is None or txt.strip() == ""



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'CssClass.category'
        db.add_column('semanticeditor_cssclass', 'category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['semanticeditor.CssClassCategory'], null=True, blank=True), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'CssClass.category'
        db.delete_column('semanticeditor_cssclass', 'category_id')


    models = {
        'semanticeditor.cssclass': {
            'Meta': {'ordering': "('verbose_name',)", 'object_name': 'CssClass'},
            'allowed_elements': ('django.db.models.fields.CharField', [], {'default': "'h1 h2 h3 h4 h5 h6 p blockquote ul ol li newrow newcol'", 'max_length': '255'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['semanticeditor.CssClassCategory']", 'null': 'True', 'blank': 'True'}),
            'column_equiv': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'templates': ('semanticeditor.fields.MultiSelectField', [], {'default': "''", 'blank': 'True'}),
            'verbose_name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'semanticeditor.cssclasscategory': {
            'Meta': {'ordering': "('name',)", 'object_name': 'CssClassCategory'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['semanticeditor']
